constructors <- local({
    idTypes <- names(slot(getClass("GeneIdentifierType"), "subclasses"))
    idTypes[!idTypes %in% c("AnnotationIdentifier", "KEGGFrameIdentifier")]
})

test_GeneIdentifierType_Constructors <- function() {
    ## do they exist and return the correct class?
    frameIds <- paste0("GOAll", "FrameIdentifier")
    for (i in seq_along(constructors)) {
        res <- do.call(constructors[[i]], list())
        checkTrue(validObject(res))
        checkTrue(is(res, constructors[[i]]))
        if (!constructors[[i]] %in% frameIds) {
            res <- do.call(constructors[[i]], list("foo"))
            checkTrue(validObject(res))
            checkTrue(is(res, constructors[[i]]))
        }
    }

    ## Required slot for AnnotationIdentifier
    res <- AnnotationIdentifier(annotation="123")
    checkTrue(validObject(res))
    checkTrue(is(res, "AnnotationIdentifier"))
}

test_GeneIdentifierType_mapIdentifiers_AnnDbBimap <- function() {
    library(org.Hs.eg.db)
    gs <- mapIdentifiers(GeneSet("4214"), SymbolIdentifier(),
                         org.Hs.egSYMBOL)
    checkIdentical("MAP3K1", geneIds(gs))
    checkIdentical("Symbol", geneIdType(geneIdType(gs)))
}


test_GeneIdentifierType_mapIdentifiers_selectMaps <- function() {
    f <- GSEABase:::.mapIdentifiers_selectMaps
    ## all fully specified and mappable at this point
    ## single maps
    chk1 <- function(map, id1, id2) {
        res <- f(id1, id2)
        checkTrue(length(res)==1)
        all.equal(map, res[[1]])
    }
    ## two maps
    chk2 <- function(map1, map2, id1, id2) {
        res <- f(id1, id2)
        checkTrue(length(res)==2)
        all.equal(map1, res[[1]])
        all.equal(map2, res[[2]])
    }

}


## edge cases

test_GeneIdentifierType_mapIdentifiers_nullAmbiguity <- function() {
    ## Original bug: 
    ##     1: Ambiguous method selection for "mapIdentifiers", target "GeneSet#AnnotationIdentifier#NullIdentifier" (the first of the signatures shown will be used)
    ##     GeneSet#AnnotationIdentifier#GeneIdentifierType
    ##     GeneSet#GeneIdentifierType#NullIdentifier
    opts <- options(warn=2)
    on.exit(options(opts))
    gs <- GeneSet(setName="123", setIdentifier="345")
    geneIdType(gs) <- AnnotationIdentifier("xyz")
    checkTrue(validObject(gs))
}

